@@ -12,8 +12,10 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <sys/stat.h>
 
 #include <errno.h>
 
@@ -212,7 +214,6 @@ static void create_appledesktop_folder(const struct vol * vol)
 {
     bstring olddtpath = NULL, dtpath = NULL;
     struct stat st;
-    char *cmd_argv[4];
 
     olddtpath = bfromcstr(vol->v_path);
     bcatcstr(olddtpath, "/" APPLEDESKTOP);
@@ -220,27 +221,24 @@ static void create_appledesktop_folder(const struct vol * vol)
     dtpath = bfromcstr(vol->v_dbpath);
     bcatcstr(dtpath, "/" APPLEDESKTOP);
 
+    become_root();
+
     if (lstat(cfrombstr(dtpath), &st) != 0) {
-
-        become_root();
-
-        if (lstat(cfrombstr(olddtpath), &st) == 0) {
-            cmd_argv[0] = "mv";
-            cmd_argv[1] = bdata(olddtpath);
-            cmd_argv[2] = bdata(dtpath);
-            cmd_argv[3] = NULL;
-            if (run_cmd("mv", cmd_argv) != 0) {
-                LOG(log_error, logtype_afpd, "moving .AppleDesktop from \"%s\" to \"%s\" failed",
+        if ((lstat(cfrombstr(olddtpath), &st) == 0) && (S_ISDIR(st.st_mode) != 0)) {
+	    if (rename(bdata(olddtpath), bdata(dtpath)) != 0) {
+                LOG(log_error, logtype_afpd, "moving .AppleDesktop from \"%s\" failed; creating new dir \"%s\"",
                     bdata(olddtpath), bdata(dtpath));
                 mkdir(cfrombstr(dtpath), 0777);
             }
         } else {
+            LOG(log_debug, logtype_afpd, "no valid .AppleDesktop dir found; creating new dir \"%s\"",
+                bdata(dtpath));
             mkdir(cfrombstr(dtpath), 0777);
         }
-
-        unbecome_root();
     }
 
+    unbecome_root();
+
     bdestroy(dtpath);
     bdestroy(olddtpath);
 }
@@ -859,7 +857,7 @@ static int ad_addcomment(const AFPObj *obj, struct vol *vol, struct path *path,
         return( AFPERR_ACCESS );
     }
 
-    if (ad_getentryoff(adp, ADEID_COMMENT)) {
+    if (ad_getentryoff(adp, ADEID_COMMENT) && ad_entry(adp, ADEID_COMMENT)) {
         if ( (ad_get_MD_flags( adp ) & O_CREAT) ) {
             if ( *path->m_name == '\0' ) {
                 name = (char *)curdir->d_m_name->data;
@@ -932,7 +930,7 @@ static int ad_getcomment(struct vol *vol, struct path *path, char *rbuf, size_t
         return( AFPERR_NOITEM );
     }
 
-    if (!ad_getentryoff(adp, ADEID_COMMENT)) {
+    if (!ad_getentryoff(adp, ADEID_COMMENT) || !ad_entry(adp, ADEID_COMMENT)) {
         ad_close(adp, ADFLAGS_HF);
         return AFPERR_NOITEM;
     }
