@@ -330,11 +330,13 @@ int ad_init_offsets(struct adouble *ad)
 
     memset(ad->ad_data, 0, sizeof(ad->ad_data));
 
-    if (ad->ad_vers == AD_VERSION2)
+    if (ad->ad_vers == AD_VERSION2) {
         eid = entry_order2;
-    else if (ad->ad_vers == AD_VERSION_EA)
+        ad->valid_data_len = AD_DATASZ2;
+    } else if (ad->ad_vers == AD_VERSION_EA) {
         eid = entry_order_ea;
-    else
+        ad->valid_data_len = AD_DATASZ_EA;
+    } else
         return -1;
 
     while (eid->id) {
@@ -371,8 +373,10 @@ static int new_ad_header(struct adouble *ad, const char *path, struct stat *stp,
         ad->valid_data_len = ad->ad_vers == AD_VERSION_EA ? AD_DATASZ_EA : AD_DATASZ2;
     }
     adp = ad_entry(ad, ADEID_FINDERI);
-    AFP_ASSERT(adp != NULL);
-
+    if (adp == NULL) {
+        LOG(log_debug, logtype_ad, "new_ad_header(\"%s\"): invalid FinderInfo", path);
+        return -1;
+    }
     /* set default creator/type fields */
     memcpy(adp + FINDERINFO_FRTYPEOFF,"\0\0\0\0", 4);
     memcpy(adp + FINDERINFO_FRCREATOFF,"\0\0\0\0", 4);
@@ -426,7 +430,7 @@ static int parse_entries(struct adouble *ad, uint16_t nentries, size_t valid_dat
 
         if (!eid
             || eid > ADEID_MAX
-            || ((eid != ADEID_RFORK) && (off >= valid_data_len))
+            || ((eid != ADEID_RFORK) && (off > valid_data_len))
             || ((eid != ADEID_RFORK) && (off + len >  valid_data_len)))
         {
             LOG(log_warning, logtype_ad, "parse_entries: bogus eid: %u, off: %u, len: %u",
@@ -624,15 +628,23 @@ static int ad_convert_osx(const char *path, struct adouble *ad)
         EC_EXIT_STATUS(0);
     struct adouble adea;
     ad_init_old(&adea, AD_VERSION_EA, ad->ad_options);
+    if (ad_init_offsets(&adea) != 0) {
+        LOG(log_error, logtype_ad, "ad_init_offsets failed");
+        EC_FAIL;
+    }
 
     if (ad_open(&adea, path + 2, ADFLAGS_HF | ADFLAGS_RDWR | ADFLAGS_CREATE, 0666) < 0) {
         LOG(log_error, logtype_ad, "create metadata: %s\n", strerror(errno));
         EC_FAIL;
     }
     if (adea.ad_mdp->adf_flags & O_CREAT) {
-        memcpy(ad_entry(&adea, ADEID_FINDERI),
-               ad_entry(ad, ADEID_FINDERI),
-               ADEDLEN_FINDERI);
+        if (ad_entry(ad, ADEID_FINDERI)) {
+            memcpy(ad_entry(&adea, ADEID_FINDERI),
+                   ad_entry(ad, ADEID_FINDERI),
+                   ADEDLEN_FINDERI);
+        } else {
+            LOG(log_debug, logtype_ad, "ad_convert_osx(%s): invalid FinderInfo", fullpathname(path));
+        }
         ad_flush(&adea);
     }
     ad_close(&adea, ADFLAGS_HF);
@@ -792,9 +804,6 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
     }
 
     /*
-     * It is possible for AFP metadata to contain a zero-length
-     * comment. This will cause ad_entry(ad, ADEID_COMMENT) to return NULL
-     * but should not be treated as an error condition.
      * Since recent CVE fixes have introduced new behavior regarding
      * ad_entry() output. For now, we will AFP_ASSERT() in EC_CLEANUP to prevent
      * altering on-disk info. This does introduce an avenue to DOS
@@ -804,6 +813,7 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
      */
     if (nentries != ADEID_NUM_EA
         || !ad_entry(ad, ADEID_FINDERI)
+        || !ad_entry(ad, ADEID_COMMENT)
         || !ad_entry(ad, ADEID_FILEDATESI)
         || !ad_entry(ad, ADEID_AFPFILEI)
         || !ad_entry(ad, ADEID_PRIVDEV)
@@ -821,12 +831,6 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
         EC_FAIL;
     }
 
-    if (!ad_entry(ad, ADEID_COMMENT) &&
-        (ad->ad_eid[ADEID_COMMENT].ade_len != 0)) {
-        errno = EINVAL;
-        EC_FAIL;
-    }
-
     /*
      * Ensure the resource fork offset is always set
      */
@@ -835,6 +839,8 @@ static int ad_header_read_ea(const char *path, struct adouble *ad, const struct
         ad_setentryoff(ad, ADEID_RFORK, ADEDOFF_RFORK_OSX);
 #endif
 
+    // TODO: Remove the assert and restore the branch for cleaning up invalid metadata
+    //       https://github.com/Netatalk/netatalk/issues/400
 EC_CLEANUP:
     AFP_ASSERT(!(ret != 0 && errno == EINVAL));
 #if 0
@@ -1673,10 +1679,14 @@ void *ad_entry(const struct adouble *ad, int eid)
 
 	valid = ad_entry_check_size(eid, bufsize, off, len);
 	if (!valid) {
+        LOG(log_debug, logtype_ad, "ad_entry(%s, %d): invalid off: %d, len: %llu, buf: %llu",
+            ad->ad_name, eid, off, len, bufsize);
 		return NULL;
 	}
 
-	if (off == 0 || len == 0) {
+	if (off == 0) {
+        LOG(log_debug, logtype_ad, "ad_entry(%s, %d): invalid off: %d, len: %llu",
+            ad->ad_name, eid, off, len);
 		return NULL;
 	}
 
